using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

// uncomment to define alias to List<int>
//using IntList = System.Collections.Generic.List<int>;

namespace r2musings.Presentations.Generics101
{
  class ArrayListTest
  {
    static void Main(string[] args)
    {
      CreateArrayListAndDisplayResults();
      Console.WriteLine();
      CreateGenericIntListAndDisplayResults();
      
      // hold console on screen
      Console.ReadLine();
    }
    
    
    private static void CreateArrayListAndDisplayResults() 
    {
      // standard Non-Generic ArrayList
      ArrayList arrayList = new ArrayList();

      // add integers (with boxing ints to objects)
      arrayList.Add(40);
      arrayList.Add(50);
      arrayList.Add(23);

      // add DateTime (boxing to object) - compiles fine
      arrayList.Add(DateTime.Now);

      Console.WriteLine("ArrayList created - now try casting each to int and displaying");
      
      // loop through and try cast to int on each arrayItem 
      foreach (object arrayItem in arrayList)
      {
        try
        {
          // this will cause an exception on anything in the arrayList
          //  that cannot be cast to an int (at runtime)
          int newInt = (int) arrayItem;

          //  casting to int before calling ToString()              
          Console.WriteLine(newInt.ToString());
        }
        catch (Exception ex)
        {
          Console.WriteLine(String.Format("{0} : {1}",
            ex.GetType().ToString(), arrayItem.ToString()));
          continue;
        }
      }
    }

    
    private static void CreateGenericIntListAndDisplayResults()  
    {
      List<int> intList = new List<int>();

      // add integers (no boxing here)
      intList.Add(4);
      intList.Add(8);
      intList.Add(12);

      // the below will not compile (type safety at compile time)  
      //intList.Add(DateTime.Now);


      Console.WriteLine("List<int> created - now displaying each with No cast");
      
      foreach (int i in intList)
      {
        // looping through and displaying without having to cast to int
        Console.WriteLine(i.ToString());
      }
    }
  }
}
