using System;
using System.Collections.Generic;
using System.Text;

namespace r2musings.Presentations.Generics101
{
  #region Person class
  class Person 
  {
    // use automatic properties (C# 3.5)
    public string FirstName { get; set; }
    public string LastName { get; set; }

    public override string ToString()
    {
      return string.Format("{0} {1}", FirstName, LastName);
    }
  }
  #endregion
  
  class Collections
  {
    const int count = 10;
    const string line = "------------------------------------------------------------";
  
    static void Main(string[] args)
    {
      CreateAndEmitList();
      CreateAndEmitDictionary();
      CreateAndEmitStack();
      CreateAndEmitQueue();

      // hold console on screen
      Console.ReadLine();
    }
    
    
    private static void CreateAndEmitList()
    {
      Console.WriteLine("\nCreate and emit List<Person>\n" + line);
    
      // create list of Persons
      List<Person> personList = new List<Person>();
      
      // loop through and add persons
      for (int i = 0; i < count; i++)
      {
        // use object initializers to create the Person
        personList.Add(new Person() { 
          FirstName = "Test", 
          LastName = String.Format("Person{0}", i) 
        } );
      }

      // print persons
      foreach (Person person in personList)
      {
        Console.WriteLine(person.ToString());
      }
    }

    private static void CreateAndEmitDictionary()
    {
      Console.WriteLine("\nCreate and emit Dictionary<int, Person>\n" + line);
    
      // create dictionary of Persons with int as Key and Person as Value
      Dictionary<int, Person> personDictionary = new Dictionary<int, Person>();
      
      // loop through and add persons
      for (int i = 0; i < count; i++)
      {
        // use object initializers to create the Person
        personDictionary.Add(i, new Person() { 
          FirstName = "Test", 
          LastName = String.Format("Person{0}", i) 
        });
      }

      // print persons  
      foreach (KeyValuePair<int, Person> kvPair in personDictionary)
      {
        Console.WriteLine(String.Format("Key: {0}  ({1})\t\tValue: {2} ({3})",
          kvPair.Key.GetType().Name, kvPair.Key.ToString(),
          kvPair.Value.GetType().Name, kvPair.Value.ToString()));
      }
    }

    private static void CreateAndEmitStack()
    {
      Console.WriteLine("\nCreate and emit Stack<Person>\n" + line);

      // create Stack of Persons
      Stack<Person> personStack = new Stack<Person>();

      // loop through and add persons
      for (int i = 0; i < count; i++)
      {
        // use object initializers to create the Person
        personStack.Push(new Person() { 
          FirstName = "Test", 
          LastName = String.Format("Person{0}", i) 
        });
      }

      // print persons  
      foreach (Person person in personStack)
      {
        Console.WriteLine(person.ToString());
      }
    }

    private static void CreateAndEmitQueue()
    {
      Console.WriteLine("\nCreate and emit Queue<Person>\n" + line);

      // create Queue of Persons
      Queue<Person> personQueue = new Queue<Person>();

      // loop through and add persons
      for (int i = 0; i < count; i++)
      {
        // use object initializers to create the Person
        personQueue.Enqueue(new Person() { 
          FirstName = "Test", 
          LastName = String.Format("Person{0}", i) 
        });        
      }

      // print persons  
      foreach (Person person in personQueue)
      {
        Console.WriteLine(person.ToString());
      }
    }

  }
 }
