using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI.WebControls;
using System.Web.UI;

namespace r2musings.Web.UI.WebControls
{
  class WebControls_Utility
  {
    /// <summary>
    ///   Searches recursively for a Control with supplied ID  
    /// </summary>
    /// <param name="root">Root Control to start search from</param>
    /// <param name="id">ID that we are looking for</param>
    /// <returns></returns>
    internal static Control FindControlRecursive(Control root, string id)
    {
      if (root.ID == id)
      {
        return root;
      }

      foreach (Control c in root.Controls)
      {
        Control t = FindControlRecursive(c, id);
        if (t != null)
        {
          return t;
        }
      }

      return null;
    }   
    
    /// <summary>
    ///   returns comma-delimited list of ClientIDs for a provided List of WebControls
    /// </summary>
    /// <param name="controls">List of WebControls to get ClientIDs for</param>
    /// <returns></returns>
    internal static string GetCommaDelimitedClientIds(List<WebControl> controls)
    {
      StringBuilder sb = new StringBuilder();

      foreach (WebControl control in controls)
      {
        // if this is not the first control, add a comma
        if (sb.Length != 0)
          sb.Append(",");

        // add the comma
        sb.Append(control.ClientID);
      }

      return sb.ToString();
    }
  }
}
