using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace r2musings.Web.UI.WebControls
{
  public static class Utility
  {
    #region FindControlsByType
    
    /// <summary>
    ///   Returns a generic list of controls of a provided type starting at a 
    ///    a provided base control (works recursively) 
    ///    Example Usage:  
    ///       List<Button> buttonList = Utility.FindControlsByType<Button>(testPanel);
    ///       This would return a list of all Buttons contained anywhere within testPanel
    /// </summary>
    /// <typeparam name="T">Type of control</typeparam>
    /// <param name="parentControl">Base control to start search</param>
    /// <returns></returns>
    public static List<T> FindControlsByType<T>(Control parentControl) where T: System.Web.UI.Control
    {
      // new up our return list
      List<T> returnList = new List<T>();

      // loop through all controls and call internal recursion to 
      //   add all controls of type T to the returnList
      foreach (Control childControl in parentControl.Controls)
      {
        InternalFindControlsByType<T>(childControl, returnList);
      }
      
      // return our List<T>
      return returnList;
    }
    
    #endregion

    #region Recursion Method
    
    /// <summary>
    ///   Should NOT call this method directly
    ///   This is for the internal recursion of FindControlsByType()  
    /// </summary>
    /// <typeparam name="T">Type of control</typeparam>
    /// <param name="parentControl">Base control to start search</param>
    /// <param name="returnList">List to add Controls</param>
    private static void InternalFindControlsByType<T>(Control parentControl, List<T> returnList) where T: System.Web.UI.Control
    {
      if (returnList == null)
        throw new ArgumentNullException("Null List passed to InternalFindControlsByType");

      if (parentControl is T)
      {
        returnList.Add((T) parentControl);
      }

      foreach (Control childControl in parentControl.Controls)
      {
        // call this method recursively to get all child controls
        InternalFindControlsByType(childControl, returnList);
      }
    }
    
    #endregion
  }
}