using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Text;
using r2musings.Web.UI.WebControls;

namespace r2musings.Presentations.Generics101
{
  public partial class FindControl : System.Web.UI.Page 
  {
    protected void Page_Load(object sender, EventArgs e)
    {
      // show the Postback time 
      lblTimestamp.Text = (Page.IsPostBack) ? 
        String.Format("Postback at: {0}", DateTime.Now.ToString()) : String.Empty;
    }

    #region Overridden methods
    protected override void OnPreRender(EventArgs e)
    {
      base.OnPreRender(e);

      // register script to validate that at least one Request type is selected    
      if (!Page.ClientScript.IsClientScriptBlockRegistered("ValidateRequestType"))
      {
        Page.ClientScript.RegisterClientScriptBlock(this.GetType(),
          "ValidateRequestType", GetRequestTypeClientValidation(), true);
      }
    }
    #endregion
   
    #region Validate Request Type
     /// <summary>
      ///   Validates that there is at least one Request Type checked in the RequestType Panel
      /// </summary>
      /// <param name="source"></param>
      /// <param name="args"></param>
      protected void valRequestType_ServerValidate(object source, ServerValidateEventArgs args)
      {
        // default isValid to false (assume no checkboxes are checked)
        args.IsValid = false;
        
        // loop through and if find one that is checked, set to true and bail
        foreach (CheckBox checkBox in Utility.FindControlsByType<CheckBox>(pnlRequestType))
        {
          if (checkBox.Checked)
          {  
            args.IsValid = true;
            break;
          }  
        }
      }


      /// <summary>
      ///   This builds the client-side function that validates that there is at least one
      ///    request type checked
      /// </summary>
      /// <returns>returns the client-side function as a string</returns>
      private string GetRequestTypeClientValidation()
      {
        StringBuilder sb = new StringBuilder();
        sb.Append("function ValidateRequestType(source, args) {");

        foreach (CheckBox checkBox in Utility.FindControlsByType<CheckBox>(pnlRequestType))
        {
          sb.AppendFormat("if (document.getElementById('{0}').checked) ", checkBox.ClientID);
          sb.Append("{ args.IsValid = true; return true; }");
        }

        sb.Append("args.IsValid = false; }");
        return sb.ToString();
      }
    #endregion
  }
}