using System;
using System.Collections.Generic;
using System.Text;

namespace r2musings.Presentations.Generics101
{
  class GenericMethods
  {
    static void Main(string[] args)
    {
      // create test ints
      int a = 5, b = 10;

      // call methods with int (explicitly specifying type parameter)
      Swap<int>(ref a, ref b);
      SwapIfGreater<int>(ref a, ref b);
      
      // call method without type param (method will infer type)
      SwapIfGreater(ref a, ref b);
      
      // create some test strings
      string s1 = "Kyle";
      string s2 = "Stan";

      // call methods with string (explicitly specifying type parameter)
      Swap<string>(ref s1, ref s2);
      SwapIfGreater<string>(ref s1, ref s2);

      // call method without type param (method will infer type)
      SwapIfGreater(ref s1, ref s2);
      
      // hold console on screen
      Console.ReadLine();
    }
    
    static void Swap<T>(ref T obj1, ref T obj2)
    {
      // log begin
      Console.WriteLine(String.Format("\nBegin Swap<{0}>: \t\t obj1 = {1}\tobj2 = {2}", 
        typeof(T).Name, obj1, obj2));
        
      // actual swap  
      T temp;
      temp = obj1;
      obj1 = obj2;
      obj2 = temp;
      
      // log end
      Console.WriteLine(String.Format("End Swap<{0}>: \t\t obj1 = {1}\tobj2 = {2}",
        typeof(T).Name, obj1, obj2));
    }
    
    // note constraint that T implement IComparable 
    static void SwapIfGreater<T>(ref T obj1, ref T obj2) where T: System.IComparable<T>
    {
      // log begin
      Console.WriteLine(String.Format("\nBegin SwapIfGreater<{0}>: \t obj1 = {1}\tobj2 = {2}",
        typeof(T).Name, obj1, obj2));
        
      // actual swap using CompareTo() which is possible 
      //  because of constraint that T : IComparable<T>
      T temp;
      if (obj1.CompareTo(obj2) > 0)
      {
        temp = obj1;
        obj1 = obj2;
        obj2 = temp;
      }
      
      // log end
      Console.WriteLine(String.Format("End SwapIfGreater<{0}>: \t obj1 = {1}\tobj2 = {2}",
        typeof(T).Name, obj1, obj2));
    }
  }
}
